/* /////////////////////////////////////////////////////////////////////////////
 * File:        findfile_sequence_test.cpp
 *
 * Purpose:     Implementation file for the findfile_sequence_test project.
 *
 * Created:     24th April 2004
 * Updated:     26th July 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage.
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission.
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <stlsoft.h>
#include <stlsoft/searchspec_sequence.hpp>

#include <inetstl.h>
#include <inetstl/session.hpp>
#include <inetstl/connection.hpp>
#include <inetstl/filesystem_traits.hpp>
//#include <inetstl/filesystem_functionals.hpp>
#include <inetstl/findfile_sequence.hpp>
#include <inetstl_functionals.h>

#include <winstl/error_desc.hpp>

#include <string>
#include <vector>

#include <windows.h>
#include <stdio.h>
#include <iostream>

using std::cout;
using std::cerr;
using std::endl;
using std::string;
using std::vector;

/* ////////////////////////////////////////////////////////////////////////// */

static void usage(int bExit, char const *reason);

/* ////////////////////////////////////////////////////////////////////////// */

void enumerate_dir_files(HINTERNET hconn, char const *dir, char const *pattern)
{
//	typedef	stlsoft::searchspec_sequence<inetstl::findfile_sequence>	sequence_t;
	typedef	inetstl::findfile_sequence									sequence_t;

//	sequence_t	fs(hconn, dir, pattern, sequence_t::find_sequence_type::files);

#if 0
	sequence_t	fs(hconn, dir, pattern, sequence_t::files);
#else /* ? 0 */
	sequence_t	fs(hconn, dir, pattern, ';', sequence_t::files);
#endif /* 0 */

	sequence_t::const_iterator	begin	=	fs.begin();
	sequence_t::const_iterator	end		=	fs.end();

	for(; begin != end; ++begin)
	{
		printf("%s\n", (*begin).get_path());
	}
}

void enumerate_dir(HINTERNET hconn, char const *dir, char const *pattern)
{
	enumerate_dir_files(hconn, dir, pattern);

	typedef	vector<string>	dirs_t;

	dirs_t	dirs;

	{
		dirs.reserve(32);

		SetLastError(ERROR_SUCCESS);

		inetstl::findfile_sequence	fs(hconn, dir, "*", inetstl::findfile_sequence::directories);

		inetstl::findfile_sequence::const_iterator	begin	=	fs.begin();
		inetstl::findfile_sequence::const_iterator	end		=	fs.end();
		DWORD										dwErr	=	::GetLastError();

		if(	ERROR_SUCCESS != dwErr &&
			ERROR_NO_MORE_FILES != dwErr)
		{

			printf("Failed to enumerate directories: %s (%d)\n", stlsoft::get_ptr(winstl::error_desc(dwErr)), dwErr);
		}

		for(; begin != end; ++begin)
		{
			dirs.push_back((*begin).get_path());
		}
	}

	dirs_t::const_iterator	begin	=	dirs.begin();
	dirs_t::const_iterator	end		=	dirs.end();

	for(; begin != end; ++begin)
	{
		enumerate_dir(hconn, (*begin).c_str(), pattern);
	}

}

int main(int argc, char **argv)
{
#if 0
	::Sleep(100000);
#endif /* 0 */

	puts("findfile_sequence_test: " __STLSOFT_COMPILER_LABEL_STRING);

	if(argc < 6)
	{
		usage(1, "Insufficient argument(s) specified");
	}
	else
	{
		// This implementation is a bit simplistic. A better version is to be
		// found in the %STLSOFT_INCLUDE%\samples directory

		char const		*host		=	argv[1];
		char const		*username	=	argv[2];
		char const		*password	=	argv[3];
		char const		*rootDir	=	argv[4];
		char const		*pattern	=	argv[5];

		try
		{
			/* . */
			inetstl::session	sess;
			inetstl::connection	conn(sess, host, INTERNET_INVALID_PORT_NUMBER, username, password, INTERNET_SERVICE_FTP, INTERNET_FLAG_PASSIVE, 0);

			enumerate_dir(conn, rootDir, pattern);
		}
		catch(inetstl::internet_exception x)
		{
			printf("Exception caught: %s (%d, %s)\n", x.what(), x.error(), stlsoft::get_ptr(winstl::error_desc(x.error(), "wininet")));
		}
	}

    return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */

static void usage(int bExit, char const *reason)
{
	fprintf(stderr, "  USAGE: <host> <username> <password> <rootDir> <pattern>\n");
	fprintf(stderr, "\n");

	if(bExit)
	{
		exit(EXIT_FAILURE);
	}
}

/* ////////////////////////////////////////////////////////////////////////// */
